#! /usr/bin/perl

###############################################################################
# Thomas Roth, 29.1.2005
# Perl script to convert Hugin.pto file (created by autopano-sift) to PTMac file .ptm
#
# Changes:
# 1.11.2005 Version 1.1
# - Sequence of images/control points maintained
# 2.01.2006 Version 1.2
# - Changed PTMac suffix from .ptmac to .ptm throughout

###############################################################################
###############################################################################

# Include libraries
use Cwd;		# To find current directory
use strict;		# Disenable automatic variables
use File::Basename;

# Main rountine
###############################################################################
{	
	# Print usage
	if ($#ARGV < 1) {               # Print usage if no arguments given
    		print "Usage: perl pto2ptmac.pl Hugin.pto PTMacProjectTemplate.ptm\n";
    		exit 0;
	}

	# Get data from .pto file
	my $ptoFilePath=$ARGV[0];

	# Open & read .pto file
	open(PTOFILE,'<'.$ptoFilePath)or
			die("Cannot open $ptoFilePath to read.");
	my $ptoFileText;
	read PTOFILE,$ptoFileText,-s $ptoFilePath;
	close PTOFILE;

	# Open & read .ptmac template file
	my $ptmTemplateFilePath=$ARGV[1];
	open(PTMACTEMPLATE,'<'.$ptmTemplateFilePath)or
			die("Cannot open $ptmTemplateFilePath to read.");
	my $ptmTemplateFileText;
	read PTMACTEMPLATE,$ptmTemplateFileText,-s $ptmTemplateFilePath;
	close PTMACTEMPLATE;

	# Determine parent directory of PTO File (which should contain the location name used for the ptmac file)
	my ($parentDirectoryName) = ($ptoFilePath =~ m|[/\\:]([^/\\:]+)[/\\:][^/\\:]+$|);

	# Calculate path/name for .ptmac file
	my $ptmFilePath=dirname($ptoFilePath).'/'.$parentDirectoryName.'.ptm';

	# Print informational messages
	print "\n**** pto2ptmac Conversion ****\n";
	print "Source .pto file: $ptoFilePath\n.ptm template: $ptmTemplateFilePath\nDestination .ptm file: $ptmFilePath\n";


	# Extract image names, image width & height and build .ptmac source image section
	my @sourceImageLines;
	my $line;
	my $cntr = 0;
	
	while($ptoFileText =~/i w([0-9]+) h([0-9]+) .*n("[^ \t\n\r\f]+")/g) 
	{	
		my $width = $1;
		my $height = $2;
		my $imagePath = $3;
		$sourceImageLines[$cntr] = "#-imgfile $width $height 0 " . $imagePath . "\no f3 y0 r0 p0 v=0 a=0 b=0 c=0 d=0 e=0 g=0 t=0\n";
		$cntr = $cntr + 1;
	}

	print "$cntr Images found\n";

	my $sourceImageSection;

	for($line=0 ; $line< $cntr ; $line++)
	{
		$sourceImageSection = $sourceImageSection . "$sourceImageLines[$line]";
	}

	# Extract control point lines and build .ptmac control point section
	my @controlPointLines;
	$cntr = 0;
	while($ptoFileText=~/(c .*\n)/g)
	{	my $ControlPointLine = $1;
		$controlPointLines[$cntr] = $ControlPointLine . "# Control Point No " . $cntr . ": -1\n";
		$cntr = $cntr + 1;
	}

	print "$cntr Control Point Lines found\n"; 
	
	my $controlPointSection;
	for($line=0 ; $line< $cntr ; $line++)
	{
		$controlPointSection = $controlPointSection . "$controlPointLines[$line]";
	}


	# Build and write .ptm file
	my $ptmFileText;
	$ptmFileText = $ptmTemplateFileText;

	# Replace respective placeholders
	$ptmFileText =~ s/(<Control Points>)/$controlPointSection/g;
	$ptmFileText =~ s/(<Images>)/$sourceImageSection/g;

	open(PTMACFILE,'>'.$ptmFilePath)or
			die("Cannot open $ptmFilePath to write.");
	print PTMACFILE "$ptmFileText";
	close PTMACFILE;}

###############################################################################
